﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("Up/Down Walk Calculator", "Calculates characteristics from an up/down walk.")]
  [StorableClass]
  public sealed class UpDownWalkCalculator : AlgorithmCharacteristicCalculator<UpDownWalk> {

    [StorableConstructor]
    private UpDownWalkCalculator(bool deserializing) : base(deserializing) { }
    private UpDownWalkCalculator(UpDownWalkCalculator original, Cloner cloner) : base(original, cloner) { }
    public UpDownWalkCalculator() : this(new UpDownWalk()) { }
    public UpDownWalkCalculator(UpDownWalk walker) : base(walker) {
      Name = ItemName;
      Description = ItemDescription;
      characteristics = new CheckedItemList<StringValue>(
        new[] { "AutoCorrelation1", "CorrelationLength", "InformationContent",
        "PartialInformationContent", "DensityBasinInformation", "InformationStability", 
        "Diversity", "Regularity", "TotalEntropy", "PeakInformationContent",
        "PeakDensityBasinInformation", "DownWalkLength", "UpWalkLength", "UpWalkLenVar",
        "DownWalkLenVar", "LowerVariance", "UpperVariance" }.Select(x => new StringValue(x)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new UpDownWalkCalculator(this, cloner);
    }

    public override bool CanCalculate() {
      return base.CanCalculate() && Problem.Operators.Any(x => x is IManipulator);
    }

    public override IEnumerable<IResult> Calculate() {
      foreach (var result in base.Calculate())
        yield return new Result("UpDownWalk." + Algorithm.MutatorParameter.Value.Name + "." + result.Name, result.Value);
    }
  }
}
